#ifndef CON_AWF1MATC_H
#define CON_AWF1MATC_H

#include "genfile.h"
#include "lewis_schrefler.h"
#include "van_genuchten.h"

/**
   This class defines model for water flow in soils (deformable porous medium)
*/

class con_awf1matc
{
 public:
  con_awf1matc();    //constructor
  ~con_awf1matc();   //destructor

  void read(XFILE *in);
  void print(FILE *out);

  void matcond_u (matrix &d,long ri,long ci,long ipp);
  void matcap_u (matrix &d,long ri,long ci,long ipp);
  void matcond1d_u (matrix &d,long ri,long ci,long ipp);
  void matcond2d_u (matrix &d,long ri,long ci,long ipp);
  void matcond2d_ax_u (matrix &d,long ri,long ci,long ipp);
  void matcond3d_u (matrix &d,long ri,long ci,long ipp);
  void matcap1d_u (matrix &d,long ri,long ci,long ipp);
  void matcap2d_u (matrix &d,long ri,long ci,long ipp);
  void matcap2d_ax_u (matrix &d,long ri,long ci,long ipp);
  void matcap3d_u (matrix &d,long ri,long ci,long ipp);

  void matcond_l (matrix &d,long ri,long ci,long ipp);
  void matcap_l (matrix &d,long ri,long ci,long ipp);
  void matcond1d_l (matrix &d,long ri,long ci,long ipp);
  void matcond2d_l (matrix &d,long ri,long ci,long ipp);
  void matcond2d_ax_l (matrix &d,long ri,long ci,long ipp);
  void matcond3d_l (matrix &d,long ri,long ci,long ipp);
  void matcap1d_l (matrix &d,long ri,long ci,long ipp);
  void matcap2d_l (matrix &d,long ri,long ci,long ipp);
  void matcap2d_ax_l (matrix &d,long ri,long ci,long ipp);
  void matcap3d_l (matrix &d,long ri,long ci,long ipp);
  
  void rhs_u1 (matrix &d,long ri,long ci,long ipp);
  void rhs1d1 (matrix &d,long ri,long ci,long ipp);
  void rhs2d1 (matrix &d,long ri,long ci,long ipp);
  void rhs2d_ax_1 (matrix &d,long ri,long ci,long ipp);
  void rhs3d1 (matrix &d,long ri,long ci,long ipp);

  double get_sw(double pw,long ipp);
  double get_phi(double pw,long ipp);
  double get_rhos(double pw,long ipp);

  double get_kuw(double pw,long ipp);
  double get_kwu(double pw,long ipp);

  double get_capuw(double pw,long ipp);
  double get_capwu(double pw,long ipp);

  double get_fu1(double pw,long ipp);

  double nu;

  /// Lewis and Schrefler's retention curve:
  lewis_reten lewis_ret;
  /// van Genuchten's retention curve:
  van_genuchten_reten van_genuchten_ret;
  
 private:
  
  waterflowmechtype model_type;
  int compress;
  double alpha,phi0,rhos;
  double ks,kw,kintr,rhow,muw0;
  
  double mefel_units; //basic units for pressures = Pa (Pascals)
};  

#endif
